﻿using AutoMapper;
using FichaBackend.Models;
using Microsoft.EntityFrameworkCore;

namespace FichaBackend.Services;

public interface IPublicDataService
{
    public Task<IEnumerable<City>> GetAllCity();
    public Task<IEnumerable<FilmDto>> GetAllFilmsInCity(string cityName);
    public Task<IEnumerable<CardQuestion>> GetAllCards();
    public Task UpdateFilmsInCity(IEnumerable<FilmDto> films);
    public bool CityExsist(string cityName);
}

public class PublicDataService : IPublicDataService
{
    private readonly DatabaseContext _databaseContext;
    public readonly IMapper _mapper;

    public PublicDataService(DatabaseContext databaseContext, IMapper mapper)
    {
        _databaseContext = databaseContext;
        _mapper = mapper;
    }

    public async Task<IEnumerable<City>> GetAllCity()
    {
        return await _databaseContext.Cities.ToListAsync();
    }

    public async Task<IEnumerable<FilmDto>> GetAllFilmsInCity(string cityName)
    {
        var films =  await _databaseContext.Films.Where(x => x.City.Name.ToLower() == cityName.ToLower()).ToListAsync();
        var destinations = _mapper.Map<List<Film>, List<FilmDto>>(films);
        return destinations;
    }

    public async Task<IEnumerable<CardQuestion>> GetAllCards()
    {
        return await _databaseContext.CardQuestions.ToListAsync();
    }

    public async Task UpdateFilmsInCity(IEnumerable<FilmDto> films)
    {
        await _databaseContext.Films.Where(x => x.City.Name == films.First().City).ForEachAsync(x => _databaseContext.Films.Remove(x));
        
        foreach (var filmDto in films)
        {
            var film = new Film();
            film.FilmName = filmDto.FilmName;
            film.Cinema = filmDto.Cinema;
            film.Price = filmDto.Price ?? 0;
            film.Genre = filmDto.Genre;
            film.Time = filmDto.Time;
            film.ImageURL = filmDto.ImageURL;
            film.Url = filmDto.Url;
            film.City = await _databaseContext.Cities.FirstAsync(x => x.Name.ToLower() == filmDto.City.ToLower());
            await _databaseContext.Films.AddAsync(film);
        }
        
        await _databaseContext.SaveChangesAsync();
    }

    public bool CityExsist(string cityName)
    {
        return _databaseContext.Cities.Any(x => x.Name.ToLower() == cityName.ToLower());
    }
}